@@ -65,7 +65,6 @@
 #define DEFAULT_PING_INTERVAL 15
 
 #define INITIAL_SETUP_USERNAME "gnome-initial-setup"
-#define INITIAL_SETUP_TRIGGER_FILE LOCALSTATEDIR "/lib/gdm/run-initial-setup"
 
 struct GdmSimpleSlavePrivate
 {
@@ -90,6 +89,8 @@ struct GdmSimpleSlavePrivate
 
         GHashTable        *open_reauthentication_requests;
 
+        GDBusProxy        *accountsservice_proxy;
+
         guint              start_session_when_ready : 1;
         guint              waiting_to_start_session : 1;
         guint              session_is_running : 1;
@@ -1168,21 +1169,27 @@ wants_autologin (GdmSimpleSlave *slave)
 static gboolean
 wants_initial_setup (GdmSimpleSlave *slave)
 {
-        gboolean enabled;
-
-        if (!g_file_test (INITIAL_SETUP_TRIGGER_FILE, G_FILE_TEST_EXISTS)) {
-                return FALSE;
-        }
+        gboolean enabled = FALSE;
 
         if (!gdm_settings_direct_get_boolean (GDM_KEY_INITIAL_SETUP_ENABLE, &enabled)) {
                 return FALSE;
         }
 
-        if (!enabled) {
-                return FALSE;
-        }
+        return enabled;
+}
 
-        return TRUE;
+static void
+setup_session (GdmSimpleSlave *slave, gboolean has_users)
+{
+        if (wants_initial_setup (slave) && !has_users) {
+                start_initial_setup (slave);
+        } else if (wants_autologin (slave)) {
+                /* Run the init script. gdmslave suspends until script has terminated */
+                gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", GDM_USERNAME);
+        } else {
+                start_greeter (slave);
+        }
+        create_new_session (slave);
 }
 
 static gboolean
@@ -1195,16 +1202,6 @@ idle_connect_to_display (GdmSimpleSlave *slave)
         res = gdm_slave_connect_to_x11_display (GDM_SLAVE (slave));
         if (res) {
                 setup_server (slave);
-
-                if (wants_initial_setup (slave)) {
-                        start_initial_setup (slave);
-                } else if (wants_autologin (slave)) {
-                        /* Run the init script. gdmslave suspends until script has terminated */
-                        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", GDM_USERNAME);
-                } else {
-                        start_greeter (slave);
-                }
-                create_new_session (slave);
         } else {
                 if (slave->priv->connection_attempts >= MAX_CONNECT_ATTEMPTS) {
                         g_warning ("Unable to connect to display after %d tries - bailing out", slave->priv->connection_attempts);
@@ -1257,6 +1254,46 @@ on_server_died (GdmServer      *server,
 #endif
 }
 
+static void
+on_list_cached_users_complete (GObject       *proxy,
+                               GAsyncResult  *result,
+                               gpointer       user_data)
+{
+        GdmSimpleSlave *slave = GDM_SIMPLE_SLAVE (user_data);
+        GVariant *call_result = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), result, NULL);
+        GVariant *user_list;
+        gboolean have_users;
+
+        if (!call_result) {
+                have_users = FALSE;
+        } else {
+                g_variant_get (call_result, "(@ao)", &user_list);
+                have_users = g_variant_n_children (user_list) > 0;
+                g_variant_unref (user_list);
+                g_variant_unref (call_result);
+        }
+                                               
+        setup_session (slave, have_users);
+}
+
+static void
+on_accountsservice_ready (GObject       *object,
+                          GAsyncResult  *result,
+                          gpointer       user_data)
+{
+        GdmSimpleSlave *slave = GDM_SIMPLE_SLAVE (user_data);
+        GError *local_error = NULL;
+
+        slave->priv->accountsservice_proxy = g_dbus_proxy_new_for_bus_finish (result, &local_error);
+        if (!slave->priv->accountsservice_proxy) {
+                g_error ("Failed to contact accountsservice: %s", local_error->message);
+        } 
+
+        g_dbus_proxy_call (slave->priv->accountsservice_proxy, "ListCachedUsers", NULL, 0, -1, NULL,
+                           on_list_cached_users_complete, slave);
+}
+                          
+
 static gboolean
 gdm_simple_slave_run (GdmSimpleSlave *slave)
 {
@@ -1303,6 +1340,14 @@ gdm_simple_slave_run (GdmSimpleSlave *slave)
                                   G_CALLBACK (on_server_ready),
                                   slave);
 
+                g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
+                                          0, NULL,
+                                          "org.freedesktop.Accounts",
+                                          "/org/freedesktop/Accounts",
+                                          "org.freedesktop.Accounts",
+                                          NULL,
+                                          on_accountsservice_ready, slave);
+                
 #ifdef WITH_PLYMOUTH
                 slave->priv->plymouth_is_running = plymouth_is_running ();
 
@@ -1493,6 +1538,8 @@ gdm_simple_slave_stop (GdmSlave *slave)
                 g_clear_object (&self->priv->server);
         }
 
+        g_clear_object (&self->priv->accountsservice_proxy);
+
         return TRUE;
 }
 
